import { Component, Input, TemplateRef, Optional, OnInit } from '@angular/core';
import { FormGroup, FormGroupName, AbstractControl, ControlContainer } from '@angular/forms';

export interface DynarowContext {
	control: AbstractControl;
	meta: StringMap;
}

@Component({
	selector: 'app-dynaform',
	templateUrl: './dynaform.component.html',
	styleUrls: ['./dynaform.component.scss']
})
export class DynaformComponent implements OnInit {

	/*
	 * DynaformComponent: <app-dynaform>
	 *
	 * USAGE:
	 *
	 * Supply component with either a FormGroup or the name of a FormGroup,
	 * the group's meta data,
	 * and, optionally, a TemplateRef used as a field row template (overrides the default template)
	 *
	 * e.g.
	 * <app-dynaform [formGroup]="myForm" [meta]="myFormMetaData"></app-dynaform>
	 * <app-dynaform formGroupName="myFormGroupName" [meta]="myFormMetaData"></app-dynaform>
	 * <app-dynaform [formGroup]="myForm" [meta]="myFormMetaData" [template]="myFieldTemplate"></app-dynaform>
	 *
	 * If supplied with just a FormGroupName it will retieve the FormGroup from the injected ControlContainer
	 *
	 */

	@Input()
	formGroup?: FormGroup;

	@Input()
	formGroupName?: FormGroupName;

	@Input()
	set control(fg: FormGroup) {
		this.formGroup = fg;
	}

	@Input()
	set meta(data) {
		this.formMetaData = data;
	}

	@Input()
	template?: TemplateRef<any>;

	formMetaData: any; // TODO: Tighten up type
	controlNames: string[];

	constructor(
		@Optional() private cc: ControlContainer
	) {}

	ngOnInit() {
		if (!this.formGroup && this.formGroupName) {
			this.formGroup = <FormGroup>this.cc.control; // Get theFormGroup from the ControlContainer
		}
		if (!this.formGroup) {
			throw new Error('Dynaform Component initialised without [formGroup] or formGroupName');
		}
		this.controlNames = Object.keys(this.formGroup.controls);
		// this.controlNames = Object.values(this.formMetaData).map((metaFoG: StringMap) => metaFoG.name);
		// console.log(this.controlNames);
		// console.log(this.formGroup);
	}

	getTemplateContext(controlName: string): DynarowContext {
		return {
			control: this.formGroup.get(controlName),
			meta: this.formMetaData[controlName]
		};
	}

	getRowClass(inputType: string): string {
		return `row-${inputType.toLowerCase().replace('component', '')}`;
	}

}
