/*

Dynaform Service, exposing 8 public methods
===========================================

build(model, meta) - takes a model and (lazy)metadata and returns a FormGroup
autoBuildFormGroup(model, meta) - synonym for autoForm
autoBuildModeledMeta(model, meta) - takes a model and (lazy)metadata and returns expanded metadata

buildFormGroup(metadata) - builds FormGroups from modelled metdata, recursively if necessary
buildFieldSpecificMeta(metadata) - use field metadta models to fill out metadata
combineModelWithMeta(model, extraMeta) - automatically generated metadata for model then combines extra metadata
combineExtraMeta(metadata, extraMeta) - combine extra metadata into metatdata, lazyly and recursively
autoMeta(model) - generate basic metadata from a raw or mapped model, recursively if necessary


NOTES
-----
This class acts as am injectable wraper around the exports of meta-utils.ts,
and creates a buildFormGroup function using the injected FormBuilder singleton


USAGE
-----

TO ADD ...


EXAMPLES
--------

Given ... TO ADD ...

*/

import { Injectable } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {
	autoMeta, combineModelWithMeta, combineExtraMeta,
	buildFieldSpecificMeta, buildFormGroupFunctionFactory
} from './meta-utils';

@Injectable()
export class DynaformService {

	protected buildFormGroup: (meta) => FormGroup;

	constructor(private fb: FormBuilder) {
		this.buildFormGroup = buildFormGroupFunctionFactory(fb);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Convenience methods combining several steps

	build(model, meta = {}): FormGroup {
		// Shortname for autoBuildFormGroup
		return this.autoBuildFormGroup(model, meta);
	}

	autoBuildFormGroup(model, meta = {}): FormGroup {
		const modelWithMeta = this.autoBuildModeledMeta(model, meta);
		return this.buildFormGroup(modelWithMeta);
	}

	autoBuildModeledMeta(model, meta = {}) {
		const modelWithMeta = this.combineModelWithMeta(model, meta);
		return this.buildFieldSpecificMeta(modelWithMeta);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Build field-type-specific metadata using the form field models (see dynaform/models)

	buildFieldSpecificMeta(meta) {
		return buildFieldSpecificMeta(meta);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Lower-level methods

	combineModelWithMeta(model, meta) {
		return combineModelWithMeta(model, meta);
	}

	combineExtraMeta(meta, extraMeta) {
		return combineExtraMeta(meta, extraMeta);
	}

	autoMeta(model) {
		return autoMeta(model);
	}

}

